--######################################################################################################
-- FILE          : E_Dx_NRZ[n]_Wrap
-- RELATED FILES :
-- DESCRIPTION   : This file is a wrapper file for the Stratix 10 E-Tile PHY in NRZ mode. 
--               : N, x1 channel PHYs are generated based on the generic entry
--               : 
--               : Different datarates are supported at compile time rather than dynamic reconfiguration
--               :
-- AUTHOR        : James Murray 
-- DEPARTMENT    : European Regional Application Engineering
-- COMPANY       : Intel PSG
--######################################################################################################

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


entity E_Dx_NRZ0_Wrap is
	generic(	NumChannels:integer:=1;
				Rate:string:="25G8";
				SIPatterns:boolean:=TRUE);						
	port(
		refclk				: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(NumChannels-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(NumChannels-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(NumChannels-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(NumChannels-1 downto 0)   := (others => '0'); 
		reconfig_clk		: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset			: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end entity E_Dx_NRZ0_Wrap;

architecture rtl of E_Dx_NRZ0_Wrap is


-- Parallel CLK Signals	
	signal tx_clkout            : std_logic_vector(NumChannels-1 downto 0);
	signal rx_clkout            : std_logic_vector(NumChannels-1 downto 0);

	signal tx_ready             : std_logic_vector(NumChannels-1 downto 0);
	
 	type tx_parallel_data80_array is array (NumChannels-1 downto 0) of std_logic_vector(79 downto 0);  
	signal tx_parallel_data80 : tx_parallel_data80_array; 

	type rx_parallel_data80_array is array (NumChannels-1 downto 0) of std_logic_vector(79 downto 0);  
	signal rx_parallel_data80 : rx_parallel_data80_array; 	

component E_Dx_9G8NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_10G1NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_10G3NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_24G3NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_25G8NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_28G3NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_USR0NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component E_Dx_USR1NRZ is
	PORT(
			rsfec_avmm2_avmmread_in    : in  std_logic                     := 'X';             -- read
			rsfec_avmm2_avmmrequest_in : in  std_logic                     := 'X';             -- waitrequest
			rsfec_avmm2_avmmwrite_in   : in  std_logic                     := 'X';             -- write
			pll_refclk0                : in  std_logic                     := 'X';             -- clk
			reconfig_write             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
			reconfig_read              : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
			reconfig_address           : in  std_logic_vector(18 downto 0) := (others => 'X'); -- address
			reconfig_writedata         : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- writedata
			reconfig_readdata          : out std_logic_vector(7 downto 0);                     -- readdata
			reconfig_waitrequest       : out std_logic_vector(0 downto 0);                     -- waitrequest
			reconfig_clk               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			reconfig_reset             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- reset
			reset                      : in  std_logic; -- reset
			rx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			rx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			rx_is_lockedtodata         : out std_logic_vector(0 downto 0);                     -- rx_is_lockedtodata
			rx_parallel_data           : out std_logic_vector(79 downto 0);                    -- rx_parallel_data
			rx_pma_ready               : out std_logic_vector(0 downto 0);                     -- rx_pma_ready
			rx_ready                   : out std_logic;                     -- rx_ready
			rx_serial_data             : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data
			rx_serial_data_n           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- rx_serial_data_n
			tx_clkout                  : out std_logic_vector(0 downto 0);                     -- clk
			tx_coreclkin               : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- clk
			tx_parallel_data           : in  std_logic_vector(79 downto 0) := (others => 'X'); -- tx_parallel_data
			tx_pma_ready               : out std_logic_vector(0 downto 0);                     -- tx_pma_ready
			tx_ready                   : out std_logic;                     -- tx_ready
			tx_serial_data             : out std_logic_vector(0 downto 0);                     -- tx_serial_data
			tx_serial_data_n           : out std_logic_vector(0 downto 0)                      -- tx_serial_data_n
		);
end component;

component DataMux80 is
	PORT(
		Reset								: in  std_logic;
		rx_parallel_datain  			: in  std_logic_vector(79 downto 0)    := (others => '0');
		rx_clk							: in  std_logic := '0';
		tx_parallel_dataout 			: out std_logic_vector(79 downto 0)    := (others => '0');
		tx_clk							: in  std_logic := '0';
		Traffic_Tx_parallel_datain	: in  std_logic_vector(79 downto 0)    := (others => '0')
		);
end component;  

component clk_measure is
	PORT(
		refclock         : in  std_logic; -- clock to be measured
		measure_clk      : in  std_logic; -- fixed reference clock to this module, eg. 100MHz
		reset            : in  std_logic; -- reset signal 
		enable	         : in  std_logic; 
		refclock_measure : out std_logic_vector(23 downto 0);
		valid            : out std_logic
		);
end component;	
                                                               
begin                                                                                                                                     
                                                           
--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
-- Instance components here
--xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
	
CH: for i in 0 to NumChannels-1 generate 

	Gen_Dx_9G8NRZ:if Rate = "9G8" generate 
		PHY: E_Dx_9G8NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_9G8NRZ;
	
	Gen_Dx_10G1NRZ:if Rate = "10G1" generate 
		PHY: E_Dx_10G1NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_10G1NRZ;	
	
	Gen_Dx_10G3NRZ:if Rate = "10G3" generate 
		PHY: E_Dx_10G3NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_10G3NRZ;
	
	Gen_Dx_24G3NRZ:if Rate = "24G3" generate 
		PHY: E_Dx_24G3NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_24G3NRZ;	
	
	Gen_Dx_25G8NRZ:if Rate = "25G8" generate 
		PHY: E_Dx_25G8NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_25G8NRZ;
	
	Gen_Dx_28G3NRZ:if Rate = "28G3" generate 
		PHY: E_Dx_28G3NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_28G3NRZ;	
	
	Gen_Dx_USR0NRZ:if Rate = "USR0" generate 
		PHY: E_Dx_USR0NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => tx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_USR0NRZ;
	
	Gen_Dx_USR1NRZ:if Rate = "USR1" generate 
		PHY: E_Dx_USR1NRZ
			port map(   
				rsfec_avmm2_avmmread_in    => '0',			
				rsfec_avmm2_avmmrequest_in => '0',
				rsfec_avmm2_avmmwrite_in   => '0',
				pll_refclk0                => refclk(0),			
				reconfig_write             => (others => '0'),			
				reconfig_read              => (others => '0'),			
				reconfig_address           => (others => '0'),			
				reconfig_writedata         => (others => '0'),			
				reconfig_readdata          => open,			
				reconfig_waitrequest       => open,			
				reconfig_clk               => reconfig_clk,							
				reconfig_reset             => reconfig_reset,							
				reset                      => phy_reset(0),							
				rx_clkout                  => rx_clkout(i downto i),						
				rx_coreclkin               => rx_clkout(i downto i),						
				rx_is_lockedtodata         => open,									
				rx_parallel_data           => rx_parallel_data80(i),									
				rx_pma_ready               => open,									
				rx_ready                   => open,									
				rx_serial_data             => rx_serial_data(i downto i),				
				rx_serial_data_n           => rx_serial_data_n(i downto i),			
				tx_clkout                  => tx_clkout(i downto i),						
				tx_coreclkin               => tx_clkout(i downto i),						
				tx_parallel_data           => rx_parallel_data80(i),			
				tx_pma_ready               => open,									
				tx_ready                   => tx_ready(i),									
				tx_serial_data             => tx_serial_data(i downto i),				
				tx_serial_data_n           => tx_serial_data_n(i downto i)			
				);
	end generate Gen_Dx_USR1NRZ;

	Gen_Mux:if SIPatterns = TRUE generate 		
		DataPathSel: DataMux80
			port map(   
				Reset									=> tx_ready(i),
				rx_parallel_datain   			=> rx_parallel_data80(i),
				rx_clk								=> rx_clkout(i),
				tx_parallel_dataout				=> tx_parallel_data80(i),
				tx_clk								=> tx_clkout(i),
				Traffic_Tx_parallel_datain		=> x"BCBC" & x"BCBC" & x"BCBC" & x"BCBC" & x"BCBC"
				);
	end generate Gen_Mux;		
		
end generate;     

measure_txclkout0_i:clk_measure
	port map(
		refclock         => tx_clkout(0),			--: in  std_logic; // clock to be measured
		measure_clk      => reconfig_clk(0),		--: in  std_logic; // fixed reference clock to this module, eg. 100MHz
		reset            => reconfig_reset(0),		--: in  std_logic; // reset signal 
		enable	         => '1',
		refclock_measure => open,
		valid            => open
		);
		
measure_rxclkout0_i:clk_measure
	port map(
		refclock         => rx_clkout(0),			--: in  std_logic; // clock to be measured
		measure_clk      => reconfig_clk(0),		--: in  std_logic; // fixed reference clock to this module, eg. 100MHz
		reset            => reconfig_reset(0),		--: in  std_logic; // reset signal 
		enable	         => '1',
		refclock_measure => open,
		valid            => open
		);

end rtl;
