--######################################################################################################
-- FILE          : refclk_select
-- RELATED FILES :
-- DESCRIPTION   : This is just a great big generate block.
--               : I dont want physical RTL (mux) on the REFCLK because that gets implemented in the fabric and adds jitter. 
--               : A generate statement is used to select the REFCLK for each group 
--               : 
--               : 
--               :
-- AUTHOR        : James Murray 
-- DEPARTMENT    : European Regional Application Engineering
-- COMPANY       : Intel PSG
--######################################################################################################

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity E_refclk_select is
	generic(E_Dx_NRZ0_refclk_sel, E_Dx_NRZ1_refclk_sel, E_Dx_NRZ2_refclk_sel, E_Dx_NRZ3_refclk_sel, E_Dx_NRZ4_refclk_sel, E_Dx_NRZ5_refclk_sel, E_Dx_NRZ6_refclk_sel, E_Dx_NRZ7_refclk_sel :string:="R4NT";
			    E_Dx_PAM0_refclk_sel, E_Dx_PAM1_refclk_sel, E_Dx_PAM2_refclk_sel, E_Dx_PAM3_refclk_sel, E_Dx_PAM4_refclk_sel, E_Dx_PAM5_refclk_sel, E_Dx_PAM6_refclk_sel, E_Dx_PAM7_refclk_sel :string:="R4NT");
	port(           
			REFCLK_R9A0, REFCLK_R9A1, REFCLK_R9A2, REFCLK_R9A3, REFCLK_R9A4, REFCLK_R9A5, REFCLK_R9A6, REFCLK_R9A7, REFCLK_R9A8: in std_logic_vector(0 downto 0);		-- Right bank 9A
			REFCLK_R9B0, REFCLK_R9B1, REFCLK_R9B2, REFCLK_R9B3, REFCLK_R9B4, REFCLK_R9B5, REFCLK_R9B6, REFCLK_R9B7, REFCLK_R9B8: in std_logic_vector(0 downto 0);		-- Right bank 9B					
			REFCLK_R9C0, REFCLK_R9C1, REFCLK_R9C2, REFCLK_R9C3, REFCLK_R9C4, REFCLK_R9C5, REFCLK_R9C6, REFCLK_R9C7, REFCLK_R9C8: in std_logic_vector(0 downto 0);		-- Right bank 9C
			REFCLK_L8B0, REFCLK_L8B1, REFCLK_L8B2, REFCLK_L8B3, REFCLK_L8B4, REFCLK_L8B5, REFCLK_L8B6, REFCLK_L8B7, REFCLK_L8B8: in std_logic_vector(0 downto 0);		-- Left bank 8B					
			REFCLK_L8C0, REFCLK_L8C1, REFCLK_L8C2, REFCLK_L8C3, REFCLK_L8C4, REFCLK_L8C5, REFCLK_L8C6, REFCLK_L8C7, REFCLK_L8C8: in std_logic_vector(0 downto 0);		-- Left bank 8C
			
			E_Dx_NRZ0_refclk, E_Dx_NRZ1_refclk, E_Dx_NRZ2_refclk, E_Dx_NRZ3_refclk, E_Dx_NRZ4_refclk, E_Dx_NRZ5_refclk, E_Dx_NRZ6_refclk, E_Dx_NRZ7_refclk : out std_logic_vector(0 downto 0);
			E_Dx_PAM0_refclk, E_Dx_PAM1_refclk, E_Dx_PAM2_refclk, E_Dx_PAM3_refclk, E_Dx_PAM4_refclk, E_Dx_PAM5_refclk, E_Dx_PAM6_refclk, E_Dx_PAM7_refclk : out std_logic_vector(0 downto 0)
	);
end entity E_refclk_select;


architecture rtl of E_refclk_select is


begin
--##############################################################
--## E_Dx_NRZ0_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ0_refclk:
-- Bottom right tile.
	if E_Dx_NRZ0_refclk_sel = "R9A0" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A1" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A2" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A3" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A4" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A5" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A6" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A7" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ0_refclk_sel = "R9A8" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ0_refclk_sel = "R9B0" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B1" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B2" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B3" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B4" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B5" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B6" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B7" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ0_refclk_sel = "R9B8" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ0_refclk_sel = "R9C0" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C1" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C2" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C3" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C4" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C5" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C6" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C7" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ0_refclk_sel = "R9C8" generate
		E_Dx_NRZ0_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ0_refclk_sel = "L8B0" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B1" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B2" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B3" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B4" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B5" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B6" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B7" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ0_refclk_sel = "L8B8" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ0_refclk_sel = "L8C0" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C1" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C2" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C3" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C4" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C5" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C6" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C7" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ0_refclk_sel = "L8C8" generate
		E_Dx_NRZ0_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ0_refclk;	

--##############################################################
--## E_Dx_NRZ1_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ1_refclk:
-- Bottom right tile.
	if E_Dx_NRZ1_refclk_sel = "R9A0" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A1" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A2" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A3" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A4" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A5" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A6" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A7" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ1_refclk_sel = "R9A8" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ1_refclk_sel = "R9B0" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B1" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B2" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B3" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B4" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B5" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B6" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B7" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ1_refclk_sel = "R9B8" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ1_refclk_sel = "R9C0" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C1" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C2" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C3" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C4" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C5" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C6" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C7" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ1_refclk_sel = "R9C8" generate
		E_Dx_NRZ1_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ1_refclk_sel = "L8B0" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B1" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B2" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B3" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B4" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B5" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B6" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B7" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ1_refclk_sel = "L8B8" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ1_refclk_sel = "L8C0" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C1" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C2" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C3" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C4" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C5" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C6" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C7" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ1_refclk_sel = "L8C8" generate
		E_Dx_NRZ1_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ1_refclk;

--##############################################################
--## E_Dx_NRZ2_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ2_refclk:
-- Bottom right tile.
	if E_Dx_NRZ2_refclk_sel = "R9A0" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A1" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A2" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A3" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A4" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A5" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A6" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A7" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ2_refclk_sel = "R9A8" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ2_refclk_sel = "R9B0" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B1" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B2" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B3" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B4" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B5" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B6" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B7" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ2_refclk_sel = "R9B8" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ2_refclk_sel = "R9C0" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C1" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C2" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C3" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C4" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C5" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C6" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C7" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ2_refclk_sel = "R9C8" generate
		E_Dx_NRZ2_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ2_refclk_sel = "L8B0" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B1" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B2" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B3" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B4" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B5" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B6" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B7" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ2_refclk_sel = "L8B8" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ2_refclk_sel = "L8C0" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C1" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C2" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C3" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C4" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C5" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C6" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C7" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ2_refclk_sel = "L8C8" generate
		E_Dx_NRZ2_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ2_refclk;		

--##############################################################
--## E_Dx_NRZ3_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ3_refclk:
-- Bottom right tile.
	if E_Dx_NRZ3_refclk_sel = "R9A0" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A1" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A2" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A3" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A4" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A5" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A6" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A7" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ3_refclk_sel = "R9A8" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ3_refclk_sel = "R9B0" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B1" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B2" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B3" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B4" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B5" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B6" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B7" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ3_refclk_sel = "R9B8" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ3_refclk_sel = "R9C0" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C1" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C2" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C3" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C4" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C5" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C6" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C7" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ3_refclk_sel = "R9C8" generate
		E_Dx_NRZ3_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ3_refclk_sel = "L8B0" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B1" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B2" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B3" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B4" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B5" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B6" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B7" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ3_refclk_sel = "L8B8" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ3_refclk_sel = "L8C0" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C1" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C2" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C3" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C4" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C5" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C6" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C7" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ3_refclk_sel = "L8C8" generate
		E_Dx_NRZ3_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ3_refclk;	

--##############################################################
--## E_Dx_NRZ4_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ4_refclk:
-- Bottom right tile.
	if E_Dx_NRZ4_refclk_sel = "R9A0" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A1" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A2" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A3" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A4" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A5" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A6" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A7" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ4_refclk_sel = "R9A8" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ4_refclk_sel = "R9B0" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B1" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B2" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B3" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B4" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B5" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B6" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B7" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ4_refclk_sel = "R9B8" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ4_refclk_sel = "R9C0" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C1" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C2" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C3" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C4" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C5" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C6" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C7" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ4_refclk_sel = "R9C8" generate
		E_Dx_NRZ4_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ4_refclk_sel = "L8B0" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B1" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B2" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B3" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B4" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B5" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B6" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B7" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ4_refclk_sel = "L8B8" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ4_refclk_sel = "L8C0" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C1" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C2" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C3" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C4" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C5" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C6" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C7" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ4_refclk_sel = "L8C8" generate
		E_Dx_NRZ4_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ4_refclk;	

--##############################################################
--## E_Dx_NRZ5_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ5_refclk:
-- Bottom right tile.
	if E_Dx_NRZ5_refclk_sel = "R9A0" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A1" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A2" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A3" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A4" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A5" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A6" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A7" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ5_refclk_sel = "R9A8" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ5_refclk_sel = "R9B0" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B1" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B2" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B3" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B4" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B5" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B6" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B7" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ5_refclk_sel = "R9B8" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ5_refclk_sel = "R9C0" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C1" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C2" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C3" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C4" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C5" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C6" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C7" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ5_refclk_sel = "R9C8" generate
		E_Dx_NRZ5_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ5_refclk_sel = "L8B0" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B1" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B2" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B3" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B4" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B5" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B6" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B7" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ5_refclk_sel = "L8B8" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ5_refclk_sel = "L8C0" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C1" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C2" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C3" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C4" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C5" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C6" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C7" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ5_refclk_sel = "L8C8" generate
		E_Dx_NRZ5_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ5_refclk;	

--##############################################################
--## E_Dx_NRZ6_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ6_refclk:
-- Bottom right tile.
	if E_Dx_NRZ6_refclk_sel = "R9A0" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A1" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A2" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A3" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A4" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A5" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A6" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A7" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ6_refclk_sel = "R9A8" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ6_refclk_sel = "R9B0" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B1" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B2" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B3" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B4" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B5" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B6" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B7" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ6_refclk_sel = "R9B8" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ6_refclk_sel = "R9C0" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C1" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C2" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C3" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C4" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C5" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C6" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C7" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ6_refclk_sel = "R9C8" generate
		E_Dx_NRZ6_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ6_refclk_sel = "L8B0" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B1" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B2" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B3" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B4" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B5" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B6" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B7" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ6_refclk_sel = "L8B8" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ6_refclk_sel = "L8C0" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C1" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C2" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C3" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C4" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C5" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C6" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C7" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ6_refclk_sel = "L8C8" generate
		E_Dx_NRZ6_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ6_refclk;	

--##############################################################
--## E_Dx_NRZ7_refclk_select  ##################################
--##############################################################

Gen_E_Dx_NRZ7_refclk:
-- Bottom right tile.
	if E_Dx_NRZ7_refclk_sel = "R9A0" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A0; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A1" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A1; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A2" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A2; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A3" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A3; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A4" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A4; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A5" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A5; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A6" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A6; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A7" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A7; 
	elsif E_Dx_NRZ7_refclk_sel = "R9A8" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_NRZ7_refclk_sel = "R9B0" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B0; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B1" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B1; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B2" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B2; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B3" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B3; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B4" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B4; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B5" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B5; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B6" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B6; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B7" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B7; 
	elsif E_Dx_NRZ7_refclk_sel = "R9B8" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_NRZ7_refclk_sel = "R9C0" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C0; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C1" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C1; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C2" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C2; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C3" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C3; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C4" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C4; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C5" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C5; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C6" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C6; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C7" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C7; 
	elsif E_Dx_NRZ7_refclk_sel = "R9C8" generate
		E_Dx_NRZ7_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_NRZ7_refclk_sel = "L8B0" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B0; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B1" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B1; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B2" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B2; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B3" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B3; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B4" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B4; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B5" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B5; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B6" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B6; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B7" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B7; 
	elsif E_Dx_NRZ7_refclk_sel = "L8B8" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_NRZ7_refclk_sel = "L8C0" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C0; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C1" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C1; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C2" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C2; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C3" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C3; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C4" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C4; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C5" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C5; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C6" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C6; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C7" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C7; 
	elsif E_Dx_NRZ7_refclk_sel = "L8C8" generate
		E_Dx_NRZ7_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_NRZ7_refclk;	


--##############################################################
--## E_Dx_PAM0_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM0_refclk:
-- Bottom right tile.
	if E_Dx_PAM0_refclk_sel = "R9A0" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM0_refclk_sel = "R9A1" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM0_refclk_sel = "R9A2" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM0_refclk_sel = "R9A3" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM0_refclk_sel = "R9A4" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM0_refclk_sel = "R9A5" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM0_refclk_sel = "R9A6" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM0_refclk_sel = "R9A7" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM0_refclk_sel = "R9A8" generate
		E_Dx_PAM0_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM0_refclk_sel = "R9B0" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM0_refclk_sel = "R9B1" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM0_refclk_sel = "R9B2" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM0_refclk_sel = "R9B3" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM0_refclk_sel = "R9B4" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM0_refclk_sel = "R9B5" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM0_refclk_sel = "R9B6" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM0_refclk_sel = "R9B7" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM0_refclk_sel = "R9B8" generate
		E_Dx_PAM0_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM0_refclk_sel = "R9C0" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM0_refclk_sel = "R9C1" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM0_refclk_sel = "R9C2" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM0_refclk_sel = "R9C3" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM0_refclk_sel = "R9C4" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM0_refclk_sel = "R9C5" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM0_refclk_sel = "R9C6" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM0_refclk_sel = "R9C7" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM0_refclk_sel = "R9C8" generate
		E_Dx_PAM0_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM0_refclk_sel = "L8B0" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM0_refclk_sel = "L8B1" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM0_refclk_sel = "L8B2" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM0_refclk_sel = "L8B3" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM0_refclk_sel = "L8B4" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM0_refclk_sel = "L8B5" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM0_refclk_sel = "L8B6" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM0_refclk_sel = "L8B7" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM0_refclk_sel = "L8B8" generate
		E_Dx_PAM0_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM0_refclk_sel = "L8C0" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM0_refclk_sel = "L8C1" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM0_refclk_sel = "L8C2" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM0_refclk_sel = "L8C3" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM0_refclk_sel = "L8C4" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM0_refclk_sel = "L8C5" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM0_refclk_sel = "L8C6" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM0_refclk_sel = "L8C7" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM0_refclk_sel = "L8C8" generate
		E_Dx_PAM0_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM0_refclk;	

--##############################################################
--## E_Dx_PAM1_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM1_refclk:
-- Bottom right tile.
	if E_Dx_PAM1_refclk_sel = "R9A0" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM1_refclk_sel = "R9A1" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM1_refclk_sel = "R9A2" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM1_refclk_sel = "R9A3" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM1_refclk_sel = "R9A4" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM1_refclk_sel = "R9A5" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM1_refclk_sel = "R9A6" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM1_refclk_sel = "R9A7" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM1_refclk_sel = "R9A8" generate
		E_Dx_PAM1_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM1_refclk_sel = "R9B0" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM1_refclk_sel = "R9B1" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM1_refclk_sel = "R9B2" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM1_refclk_sel = "R9B3" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM1_refclk_sel = "R9B4" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM1_refclk_sel = "R9B5" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM1_refclk_sel = "R9B6" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM1_refclk_sel = "R9B7" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM1_refclk_sel = "R9B8" generate
		E_Dx_PAM1_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM1_refclk_sel = "R9C0" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM1_refclk_sel = "R9C1" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM1_refclk_sel = "R9C2" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM1_refclk_sel = "R9C3" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM1_refclk_sel = "R9C4" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM1_refclk_sel = "R9C5" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM1_refclk_sel = "R9C6" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM1_refclk_sel = "R9C7" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM1_refclk_sel = "R9C8" generate
		E_Dx_PAM1_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM1_refclk_sel = "L8B0" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM1_refclk_sel = "L8B1" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM1_refclk_sel = "L8B2" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM1_refclk_sel = "L8B3" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM1_refclk_sel = "L8B4" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM1_refclk_sel = "L8B5" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM1_refclk_sel = "L8B6" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM1_refclk_sel = "L8B7" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM1_refclk_sel = "L8B8" generate
		E_Dx_PAM1_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM1_refclk_sel = "L8C0" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM1_refclk_sel = "L8C1" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM1_refclk_sel = "L8C2" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM1_refclk_sel = "L8C3" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM1_refclk_sel = "L8C4" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM1_refclk_sel = "L8C5" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM1_refclk_sel = "L8C6" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM1_refclk_sel = "L8C7" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM1_refclk_sel = "L8C8" generate
		E_Dx_PAM1_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM1_refclk;	

--##############################################################
--## E_Dx_PAM2_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM2_refclk:
-- Bottom right tile.
	if E_Dx_PAM2_refclk_sel = "R9A0" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM2_refclk_sel = "R9A1" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM2_refclk_sel = "R9A2" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM2_refclk_sel = "R9A3" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM2_refclk_sel = "R9A4" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM2_refclk_sel = "R9A5" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM2_refclk_sel = "R9A6" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM2_refclk_sel = "R9A7" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM2_refclk_sel = "R9A8" generate
		E_Dx_PAM2_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM2_refclk_sel = "R9B0" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM2_refclk_sel = "R9B1" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM2_refclk_sel = "R9B2" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM2_refclk_sel = "R9B3" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM2_refclk_sel = "R9B4" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM2_refclk_sel = "R9B5" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM2_refclk_sel = "R9B6" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM2_refclk_sel = "R9B7" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM2_refclk_sel = "R9B8" generate
		E_Dx_PAM2_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM2_refclk_sel = "R9C0" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM2_refclk_sel = "R9C1" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM2_refclk_sel = "R9C2" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM2_refclk_sel = "R9C3" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM2_refclk_sel = "R9C4" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM2_refclk_sel = "R9C5" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM2_refclk_sel = "R9C6" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM2_refclk_sel = "R9C7" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM2_refclk_sel = "R9C8" generate
		E_Dx_PAM2_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM2_refclk_sel = "L8B0" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM2_refclk_sel = "L8B1" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM2_refclk_sel = "L8B2" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM2_refclk_sel = "L8B3" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM2_refclk_sel = "L8B4" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM2_refclk_sel = "L8B5" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM2_refclk_sel = "L8B6" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM2_refclk_sel = "L8B7" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM2_refclk_sel = "L8B8" generate
		E_Dx_PAM2_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM2_refclk_sel = "L8C0" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM2_refclk_sel = "L8C1" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM2_refclk_sel = "L8C2" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM2_refclk_sel = "L8C3" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM2_refclk_sel = "L8C4" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM2_refclk_sel = "L8C5" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM2_refclk_sel = "L8C6" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM2_refclk_sel = "L8C7" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM2_refclk_sel = "L8C8" generate
		E_Dx_PAM2_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM2_refclk;	

--##############################################################
--## E_Dx_PAM3_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM3_refclk:
-- Bottom right tile.
	if E_Dx_PAM3_refclk_sel = "R9A0" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM3_refclk_sel = "R9A1" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM3_refclk_sel = "R9A2" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM3_refclk_sel = "R9A3" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM3_refclk_sel = "R9A4" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM3_refclk_sel = "R9A5" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM3_refclk_sel = "R9A6" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM3_refclk_sel = "R9A7" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM3_refclk_sel = "R9A8" generate
		E_Dx_PAM3_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM3_refclk_sel = "R9B0" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM3_refclk_sel = "R9B1" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM3_refclk_sel = "R9B2" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM3_refclk_sel = "R9B3" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM3_refclk_sel = "R9B4" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM3_refclk_sel = "R9B5" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM3_refclk_sel = "R9B6" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM3_refclk_sel = "R9B7" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM3_refclk_sel = "R9B8" generate
		E_Dx_PAM3_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM3_refclk_sel = "R9C0" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM3_refclk_sel = "R9C1" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM3_refclk_sel = "R9C2" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM3_refclk_sel = "R9C3" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM3_refclk_sel = "R9C4" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM3_refclk_sel = "R9C5" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM3_refclk_sel = "R9C6" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM3_refclk_sel = "R9C7" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM3_refclk_sel = "R9C8" generate
		E_Dx_PAM3_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM3_refclk_sel = "L8B0" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM3_refclk_sel = "L8B1" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM3_refclk_sel = "L8B2" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM3_refclk_sel = "L8B3" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM3_refclk_sel = "L8B4" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM3_refclk_sel = "L8B5" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM3_refclk_sel = "L8B6" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM3_refclk_sel = "L8B7" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM3_refclk_sel = "L8B8" generate
		E_Dx_PAM3_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM3_refclk_sel = "L8C0" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM3_refclk_sel = "L8C1" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM3_refclk_sel = "L8C2" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM3_refclk_sel = "L8C3" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM3_refclk_sel = "L8C4" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM3_refclk_sel = "L8C5" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM3_refclk_sel = "L8C6" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM3_refclk_sel = "L8C7" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM3_refclk_sel = "L8C8" generate
		E_Dx_PAM3_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM3_refclk;	

--##############################################################
--## E_Dx_PAM4_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM4_refclk:
-- Bottom right tile.
	if E_Dx_PAM4_refclk_sel = "R9A0" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM4_refclk_sel = "R9A1" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM4_refclk_sel = "R9A2" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM4_refclk_sel = "R9A3" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM4_refclk_sel = "R9A4" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM4_refclk_sel = "R9A5" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM4_refclk_sel = "R9A6" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM4_refclk_sel = "R9A7" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM4_refclk_sel = "R9A8" generate
		E_Dx_PAM4_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM4_refclk_sel = "R9B0" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM4_refclk_sel = "R9B1" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM4_refclk_sel = "R9B2" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM4_refclk_sel = "R9B3" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM4_refclk_sel = "R9B4" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM4_refclk_sel = "R9B5" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM4_refclk_sel = "R9B6" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM4_refclk_sel = "R9B7" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM4_refclk_sel = "R9B8" generate
		E_Dx_PAM4_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM4_refclk_sel = "R9C0" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM4_refclk_sel = "R9C1" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM4_refclk_sel = "R9C2" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM4_refclk_sel = "R9C3" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM4_refclk_sel = "R9C4" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM4_refclk_sel = "R9C5" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM4_refclk_sel = "R9C6" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM4_refclk_sel = "R9C7" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM4_refclk_sel = "R9C8" generate
		E_Dx_PAM4_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM4_refclk_sel = "L8B0" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM4_refclk_sel = "L8B1" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM4_refclk_sel = "L8B2" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM4_refclk_sel = "L8B3" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM4_refclk_sel = "L8B4" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM4_refclk_sel = "L8B5" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM4_refclk_sel = "L8B6" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM4_refclk_sel = "L8B7" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM4_refclk_sel = "L8B8" generate
		E_Dx_PAM4_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM4_refclk_sel = "L8C0" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM4_refclk_sel = "L8C1" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM4_refclk_sel = "L8C2" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM4_refclk_sel = "L8C3" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM4_refclk_sel = "L8C4" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM4_refclk_sel = "L8C5" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM4_refclk_sel = "L8C6" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM4_refclk_sel = "L8C7" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM4_refclk_sel = "L8C8" generate
		E_Dx_PAM4_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM4_refclk;	

--##############################################################
--## E_Dx_PAM5_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM5_refclk:
-- Bottom right tile.
	if E_Dx_PAM5_refclk_sel = "R9A0" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM5_refclk_sel = "R9A1" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM5_refclk_sel = "R9A2" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM5_refclk_sel = "R9A3" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM5_refclk_sel = "R9A4" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM5_refclk_sel = "R9A5" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM5_refclk_sel = "R9A6" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM5_refclk_sel = "R9A7" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM5_refclk_sel = "R9A8" generate
		E_Dx_PAM5_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM5_refclk_sel = "R9B0" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM5_refclk_sel = "R9B1" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM5_refclk_sel = "R9B2" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM5_refclk_sel = "R9B3" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM5_refclk_sel = "R9B4" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM5_refclk_sel = "R9B5" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM5_refclk_sel = "R9B6" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM5_refclk_sel = "R9B7" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM5_refclk_sel = "R9B8" generate
		E_Dx_PAM5_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM5_refclk_sel = "R9C0" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM5_refclk_sel = "R9C1" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM5_refclk_sel = "R9C2" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM5_refclk_sel = "R9C3" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM5_refclk_sel = "R9C4" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM5_refclk_sel = "R9C5" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM5_refclk_sel = "R9C6" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM5_refclk_sel = "R9C7" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM5_refclk_sel = "R9C8" generate
		E_Dx_PAM5_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM5_refclk_sel = "L8B0" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM5_refclk_sel = "L8B1" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM5_refclk_sel = "L8B2" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM5_refclk_sel = "L8B3" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM5_refclk_sel = "L8B4" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM5_refclk_sel = "L8B5" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM5_refclk_sel = "L8B6" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM5_refclk_sel = "L8B7" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM5_refclk_sel = "L8B8" generate
		E_Dx_PAM5_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM5_refclk_sel = "L8C0" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM5_refclk_sel = "L8C1" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM5_refclk_sel = "L8C2" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM5_refclk_sel = "L8C3" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM5_refclk_sel = "L8C4" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM5_refclk_sel = "L8C5" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM5_refclk_sel = "L8C6" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM5_refclk_sel = "L8C7" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM5_refclk_sel = "L8C8" generate
		E_Dx_PAM5_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM5_refclk;	

--##############################################################
--## E_Dx_PAM6_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM6_refclk:
-- Bottom right tile.
	if E_Dx_PAM6_refclk_sel = "R9A0" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM6_refclk_sel = "R9A1" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM6_refclk_sel = "R9A2" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM6_refclk_sel = "R9A3" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM6_refclk_sel = "R9A4" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM6_refclk_sel = "R9A5" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM6_refclk_sel = "R9A6" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM6_refclk_sel = "R9A7" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM6_refclk_sel = "R9A8" generate
		E_Dx_PAM6_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM6_refclk_sel = "R9B0" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM6_refclk_sel = "R9B1" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM6_refclk_sel = "R9B2" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM6_refclk_sel = "R9B3" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM6_refclk_sel = "R9B4" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM6_refclk_sel = "R9B5" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM6_refclk_sel = "R9B6" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM6_refclk_sel = "R9B7" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM6_refclk_sel = "R9B8" generate
		E_Dx_PAM6_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM6_refclk_sel = "R9C0" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM6_refclk_sel = "R9C1" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM6_refclk_sel = "R9C2" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM6_refclk_sel = "R9C3" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM6_refclk_sel = "R9C4" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM6_refclk_sel = "R9C5" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM6_refclk_sel = "R9C6" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM6_refclk_sel = "R9C7" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM6_refclk_sel = "R9C8" generate
		E_Dx_PAM6_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM6_refclk_sel = "L8B0" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM6_refclk_sel = "L8B1" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM6_refclk_sel = "L8B2" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM6_refclk_sel = "L8B3" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM6_refclk_sel = "L8B4" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM6_refclk_sel = "L8B5" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM6_refclk_sel = "L8B6" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM6_refclk_sel = "L8B7" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM6_refclk_sel = "L8B8" generate
		E_Dx_PAM6_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM6_refclk_sel = "L8C0" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM6_refclk_sel = "L8C1" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM6_refclk_sel = "L8C2" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM6_refclk_sel = "L8C3" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM6_refclk_sel = "L8C4" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM6_refclk_sel = "L8C5" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM6_refclk_sel = "L8C6" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM6_refclk_sel = "L8C7" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM6_refclk_sel = "L8C8" generate
		E_Dx_PAM6_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM6_refclk;	

--##############################################################
--## E_Dx_PAM7_refclk_select  ########################################
--##############################################################

Gen_E_Dx_PAM7_refclk:
-- Bottom right tile.
	if E_Dx_PAM7_refclk_sel = "R9A0" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A0; 
	elsif E_Dx_PAM7_refclk_sel = "R9A1" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A1; 
	elsif E_Dx_PAM7_refclk_sel = "R9A2" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A2; 
	elsif E_Dx_PAM7_refclk_sel = "R9A3" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A3; 
	elsif E_Dx_PAM7_refclk_sel = "R9A4" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A4; 
	elsif E_Dx_PAM7_refclk_sel = "R9A5" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A5; 
	elsif E_Dx_PAM7_refclk_sel = "R9A6" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A6; 
	elsif E_Dx_PAM7_refclk_sel = "R9A7" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A7; 
	elsif E_Dx_PAM7_refclk_sel = "R9A8" generate
		E_Dx_PAM7_refclk <= REFCLK_R9A8; 		

-- Middle right tile.
	elsif E_Dx_PAM7_refclk_sel = "R9B0" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B0; 
	elsif E_Dx_PAM7_refclk_sel = "R9B1" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B1; 
	elsif E_Dx_PAM7_refclk_sel = "R9B2" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B2; 
	elsif E_Dx_PAM7_refclk_sel = "R9B3" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B3; 
	elsif E_Dx_PAM7_refclk_sel = "R9B4" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B4; 
	elsif E_Dx_PAM7_refclk_sel = "R9B5" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B5; 
	elsif E_Dx_PAM7_refclk_sel = "R9B6" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B6; 
	elsif E_Dx_PAM7_refclk_sel = "R9B7" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B7; 
	elsif E_Dx_PAM7_refclk_sel = "R9B8" generate
		E_Dx_PAM7_refclk <= REFCLK_R9B8; 		

-- Top right tile.
	elsif E_Dx_PAM7_refclk_sel = "R9C0" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C0; 
	elsif E_Dx_PAM7_refclk_sel = "R9C1" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C1; 
	elsif E_Dx_PAM7_refclk_sel = "R9C2" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C2; 
	elsif E_Dx_PAM7_refclk_sel = "R9C3" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C3; 
	elsif E_Dx_PAM7_refclk_sel = "R9C4" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C4; 
	elsif E_Dx_PAM7_refclk_sel = "R9C5" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C5; 
	elsif E_Dx_PAM7_refclk_sel = "R9C6" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C6; 
	elsif E_Dx_PAM7_refclk_sel = "R9C7" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C7; 
	elsif E_Dx_PAM7_refclk_sel = "R9C8" generate
		E_Dx_PAM7_refclk <= REFCLK_R9C8;		
		
-- Middle left tile.
	elsif E_Dx_PAM7_refclk_sel = "L8B0" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B0; 
	elsif E_Dx_PAM7_refclk_sel = "L8B1" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B1; 
	elsif E_Dx_PAM7_refclk_sel = "L8B2" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B2; 
	elsif E_Dx_PAM7_refclk_sel = "L8B3" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B3; 
	elsif E_Dx_PAM7_refclk_sel = "L8B4" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B4; 
	elsif E_Dx_PAM7_refclk_sel = "L8B5" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B5; 
	elsif E_Dx_PAM7_refclk_sel = "L8B6" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B6; 
	elsif E_Dx_PAM7_refclk_sel = "L8B7" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B7; 
	elsif E_Dx_PAM7_refclk_sel = "L8B8" generate
		E_Dx_PAM7_refclk <= REFCLK_L8B8; 		

-- Top left tile.
	elsif E_Dx_PAM7_refclk_sel = "L8C0" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C0; 
	elsif E_Dx_PAM7_refclk_sel = "L8C1" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C1; 
	elsif E_Dx_PAM7_refclk_sel = "L8C2" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C2; 
	elsif E_Dx_PAM7_refclk_sel = "L8C3" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C3; 
	elsif E_Dx_PAM7_refclk_sel = "L8C4" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C4; 
	elsif E_Dx_PAM7_refclk_sel = "L8C5" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C5; 
	elsif E_Dx_PAM7_refclk_sel = "L8C6" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C6; 
	elsif E_Dx_PAM7_refclk_sel = "L8C7" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C7; 
	elsif E_Dx_PAM7_refclk_sel = "L8C8" generate
		E_Dx_PAM7_refclk <= REFCLK_L8C8;	
				
end generate Gen_E_Dx_PAM7_refclk;	

end rtl;