--######################################################################################################
-- FILE          : XCVR_Top
-- RELATED FILES :
-- DESCRIPTION   : This is the top level E-tile transceiver file
--					  : PHY wrappers are instantiated by entering >0 in the *_num Generics
--               : The *_Rate generic chooses pre-generated PHYs of various datarates
--               : The REFCLK is chosen by the *_refclk_sel generic
--               :
-- AUTHOR        : James Murray, 
-- DEPARTMENT    : European Regional Application Engineering,
-- COMPANY       : Intel PSG.
--######################################################################################################

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity XCVR_Top is
	generic(

--############################################################################
-- Duplex NRZ PHYs
--############################################################################

			E_Dx_NRZ0_num:integer:=1;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ0_Rate:string:="10G3";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ0_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ0_refclk_sel:string:="R9A1";	-- Enter REFCLK choice here.

			E_Dx_NRZ1_num:integer:=1;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ1_Rate:string:="25G8";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ1_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ1_refclk_sel:string:="R9A1";	-- Enter REFCLK choice here. 
			
			E_Dx_NRZ2_num:integer:=1;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ2_Rate:string:="28G3";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ2_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ2_refclk_sel:string:="R9A0";	-- Enter REFCLK choice here.

			E_Dx_NRZ3_num:integer:=1;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ3_Rate:string:="9G8";				-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ3_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ3_refclk_sel:string:="L8B1";	-- Enter REFCLK choice here. 			
			
			E_Dx_NRZ4_num:integer:=1;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ4_Rate:string:="10G1";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ4_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ4_refclk_sel:string:="L8B1";	-- Enter REFCLK choice here.

			E_Dx_NRZ5_num:integer:=0;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ5_Rate:string:="10G1";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ5_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ5_refclk_sel:string:="R9A2";	-- Enter REFCLK choice here. 
			
			E_Dx_NRZ6_num:integer:=0;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ6_Rate:string:="10G3";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ6_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ6_refclk_sel:string:="R9C0";	-- Enter REFCLK choice here.

			E_Dx_NRZ7_num:integer:=0;					-- Total number of Channels in this group. Enter any number from 1 to 24
			E_Dx_NRZ7_Rate:string:="24G3";			-- Enter "9G8", "10G1", "10G3", "24G3", "25G8",  "28G3","USR0", "USR1"
			E_Dx_NRZ7_EnSIPatterns:boolean:= TRUE;	-- Enter TRUE or FALSE
			E_Dx_NRZ7_refclk_sel:string:="R9A2";	-- Enter REFCLK choice here. 						
		
--############################################################################
-- Duplex PAM4 PHYs
--############################################################################
	
			E_Dx_PAM0_num:integer:=1;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM0_Rate:string:="51G5";		-- Enter "51G5", USR0, USR1
			E_Dx_PAM0_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM0_refclk_sel:string:="R9A1";	-- Enter REFCLK choice here. 

			E_Dx_PAM1_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM1_Rate:string:="USR0";		-- Enter "51G5", "53G1", USR0, USR1
			E_Dx_PAM1_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM1_refclk_sel:string:="R9B1";	-- Enter REFCLK choice here. 			
			
			E_Dx_PAM2_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM2_Rate:string:="USR1";		-- Enter "51G5", USR0, USR1
			E_Dx_PAM2_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM2_refclk_sel:string:="R9A1";	-- Enter REFCLK choice here. 

			E_Dx_PAM3_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM3_Rate:string:="USR1";		-- Enter "51G5", "53G1", USR0, USR1
			E_Dx_PAM3_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM3_refclk_sel:string:="R9B1";	-- Enter REFCLK choice here. 						
			
			E_Dx_PAM4_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM4_Rate:string:="51G5";		-- Enter "51G5", USR0, USR1
			E_Dx_PAM4_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM4_refclk_sel:string:="R9A1";	-- Enter REFCLK choice here. 

			E_Dx_PAM5_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM5_Rate:string:="USR0";		-- Enter "51G5", "53G1", USR0, USR1
			E_Dx_PAM5_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM5_refclk_sel:string:="R9B1";	-- Enter REFCLK choice here. 			
			
			E_Dx_PAM6_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM6_Rate:string:="51G5";		-- Enter "51G5", USR0, USR1
			E_Dx_PAM6_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM6_refclk_sel:string:="R9A1";	-- Enter REFCLK choice here. 

			E_Dx_PAM7_num:integer:=0;			-- Total number of Channels in this group. Enter any number from 1 to 12
			E_Dx_PAM7_Rate:string:="USR1";		-- Enter "51G5", "53G1", USR0, USR1
			E_Dx_PAM7_EnSIPatterns:boolean:= TRUE;	
			E_Dx_PAM7_refclk_sel:string:="R9B1";	-- Enter REFCLK choice here. 						
			
			UnusedGeneric:boolean:=FALSE
);	
	
	port(
	
--######################################################################
-- Reference clocks ####################################################
--######################################################################

--			Random_REFCLK	: in std_logic_vector(0 downto 0);

			REFCLK_R9A0, REFCLK_R9A1, REFCLK_R9A2, REFCLK_R9A3, REFCLK_R9A4, REFCLK_R9A5, REFCLK_R9A6, REFCLK_R9A7, REFCLK_R9A8: in std_logic_vector(0 downto 0);		-- Right bank 9A		
			REFCLK_R9B0, REFCLK_R9B1, REFCLK_R9B2, REFCLK_R9B3, REFCLK_R9B4, REFCLK_R9B5, REFCLK_R9B6, REFCLK_R9B7, REFCLK_R9B8: in std_logic_vector(0 downto 0);		-- Right bank 9B					
			REFCLK_R9C0, REFCLK_R9C1, REFCLK_R9C2, REFCLK_R9C3, REFCLK_R9C4, REFCLK_R9C5, REFCLK_R9C6, REFCLK_R9C7, REFCLK_R9C8: in std_logic_vector(0 downto 0);		-- Right bank 9C

			REFCLK_L8B0, REFCLK_L8B1, REFCLK_L8B2, REFCLK_L8B3, REFCLK_L8B4, REFCLK_L8B5, REFCLK_L8B6, REFCLK_L8B7, REFCLK_L8B8: in std_logic_vector(0 downto 0);		-- Left bank 8B					
			REFCLK_L8C0, REFCLK_L8C1, REFCLK_L8C2, REFCLK_L8C3, REFCLK_L8C4, REFCLK_L8C5, REFCLK_L8C6, REFCLK_L8C7, REFCLK_L8C8: in std_logic_vector(0 downto 0);		-- Left bank 8C
			
--######################################################################
-- NRZ Duplex Channels #############################################
--######################################################################

		Tx_E_Dx_NRZ0      	: out std_logic_vector(E_Dx_NRZ0_num-1 downto 0);
		Tx_E_Dx_NRZ0_n     	: out std_logic_vector(E_Dx_NRZ0_num-1 downto 0);		
		Rx_E_Dx_NRZ0      	: in  std_logic_vector(E_Dx_NRZ0_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ0_n     	: in  std_logic_vector(E_Dx_NRZ0_num-1 downto 0)   := (others => '0'); 	
		
		Tx_E_Dx_NRZ1      	: out std_logic_vector(E_Dx_NRZ1_num-1 downto 0);
		Tx_E_Dx_NRZ1_n     	: out std_logic_vector(E_Dx_NRZ1_num-1 downto 0);		
		Rx_E_Dx_NRZ1      	: in  std_logic_vector(E_Dx_NRZ1_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ1_n     	: in  std_logic_vector(E_Dx_NRZ1_num-1 downto 0)   := (others => '0'); 	
		
		Tx_E_Dx_NRZ2      	: out std_logic_vector(E_Dx_NRZ2_num-1 downto 0);
		Tx_E_Dx_NRZ2_n     	: out std_logic_vector(E_Dx_NRZ2_num-1 downto 0);		
		Rx_E_Dx_NRZ2      	: in  std_logic_vector(E_Dx_NRZ2_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ2_n     	: in  std_logic_vector(E_Dx_NRZ2_num-1 downto 0)   := (others => '0'); 	
		
		Tx_E_Dx_NRZ3      	: out std_logic_vector(E_Dx_NRZ3_num-1 downto 0);
		Tx_E_Dx_NRZ3_n     	: out std_logic_vector(E_Dx_NRZ3_num-1 downto 0);		
		Rx_E_Dx_NRZ3      	: in  std_logic_vector(E_Dx_NRZ3_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ3_n     	: in  std_logic_vector(E_Dx_NRZ3_num-1 downto 0)   := (others => '0'); 			
		
		Tx_E_Dx_NRZ4      	: out std_logic_vector(E_Dx_NRZ4_num-1 downto 0);
		Tx_E_Dx_NRZ4_n     	: out std_logic_vector(E_Dx_NRZ4_num-1 downto 0);		
		Rx_E_Dx_NRZ4      	: in  std_logic_vector(E_Dx_NRZ4_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ4_n     	: in  std_logic_vector(E_Dx_NRZ4_num-1 downto 0)   := (others => '0'); 	
		
		Tx_E_Dx_NRZ5      	: out std_logic_vector(E_Dx_NRZ5_num-1 downto 0);
		Tx_E_Dx_NRZ5_n     	: out std_logic_vector(E_Dx_NRZ5_num-1 downto 0);		
		Rx_E_Dx_NRZ5      	: in  std_logic_vector(E_Dx_NRZ5_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ5_n     	: in  std_logic_vector(E_Dx_NRZ5_num-1 downto 0)   := (others => '0'); 	
		
		Tx_E_Dx_NRZ6      	: out std_logic_vector(E_Dx_NRZ6_num-1 downto 0);
		Tx_E_Dx_NRZ6_n     	: out std_logic_vector(E_Dx_NRZ6_num-1 downto 0);		
		Rx_E_Dx_NRZ6      	: in  std_logic_vector(E_Dx_NRZ6_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ6_n     	: in  std_logic_vector(E_Dx_NRZ6_num-1 downto 0)   := (others => '0'); 	
		
		Tx_E_Dx_NRZ7      	: out std_logic_vector(E_Dx_NRZ7_num-1 downto 0);
		Tx_E_Dx_NRZ7_n     	: out std_logic_vector(E_Dx_NRZ7_num-1 downto 0);		
		Rx_E_Dx_NRZ7      	: in  std_logic_vector(E_Dx_NRZ7_num-1 downto 0)   := (others => '0');  
		Rx_E_Dx_NRZ7_n     	: in  std_logic_vector(E_Dx_NRZ7_num-1 downto 0)   := (others => '0'); 			
		

--######################################################################
-- PAM Duplex Channels #############################################
--######################################################################						
		                    	
		Tx_E_Dx_PAM0				  : out std_logic_vector(E_Dx_PAM0_num-1 downto 0);
		Tx_E_Dx_PAM0_n				: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);	
		Tx_E_Dx_PAM0_unused		: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);
		Tx_E_Dx_PAM0_unused_n	: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);			
		Rx_E_Dx_PAM0				  : in  std_logic_vector(E_Dx_PAM0_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM0_n				: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM0_unused		: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0);
		Rx_E_Dx_PAM0_unused_n	: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0);		
		
		Tx_E_Dx_PAM1					: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);
		Tx_E_Dx_PAM1_n				: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);	
		Tx_E_Dx_PAM1_unused		: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);
		Tx_E_Dx_PAM1_unused_n	: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);			
		Rx_E_Dx_PAM1					: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM1_n				: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM1_unused		: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0);
		Rx_E_Dx_PAM1_unused_n	: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0);		
		
		Tx_E_Dx_PAM2					: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);
		Tx_E_Dx_PAM2_n				: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);	
		Tx_E_Dx_PAM2_unused		: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);
		Tx_E_Dx_PAM2_unused_n	: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);			
		Rx_E_Dx_PAM2					: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM2_n				: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM2_unused		: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0);
		Rx_E_Dx_PAM2_unused_n	: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0);		
		
		Tx_E_Dx_PAM3					: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);
		Tx_E_Dx_PAM3_n				: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);	
		Tx_E_Dx_PAM3_unused		: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);
		Tx_E_Dx_PAM3_unused_n	: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);			
		Rx_E_Dx_PAM3					: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM3_n				: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM3_unused		: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0);
		Rx_E_Dx_PAM3_unused_n	: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0);				
		
		Tx_E_Dx_PAM4				  : out std_logic_vector(E_Dx_PAM4_num-1 downto 0);
		Tx_E_Dx_PAM4_n				: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);	
		Tx_E_Dx_PAM4_unused		: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);
		Tx_E_Dx_PAM4_unused_n	: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);			
		Rx_E_Dx_PAM4				  : in  std_logic_vector(E_Dx_PAM4_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM4_n				: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM4_unused		: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0);
		Rx_E_Dx_PAM4_unused_n	: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0);		
		
		Tx_E_Dx_PAM5					: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);
		Tx_E_Dx_PAM5_n				: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);	
		Tx_E_Dx_PAM5_unused		: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);
		Tx_E_Dx_PAM5_unused_n	: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);			
		Rx_E_Dx_PAM5					: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM5_n				: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM5_unused		: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0);
		Rx_E_Dx_PAM5_unused_n	: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0);		
		
		Tx_E_Dx_PAM6					: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);
		Tx_E_Dx_PAM6_n				: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);	
		Tx_E_Dx_PAM6_unused		: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);
		Tx_E_Dx_PAM6_unused_n	: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);			
		Rx_E_Dx_PAM6					: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM6_n				: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM6_unused		: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0);
		Rx_E_Dx_PAM6_unused_n	: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0);		
		
		Tx_E_Dx_PAM7					: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);
		Tx_E_Dx_PAM7_n				: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);	
		Tx_E_Dx_PAM7_unused		: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);
		Tx_E_Dx_PAM7_unused_n	: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);			
		Rx_E_Dx_PAM7					: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM7_n				: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0)   := (others => '0');
		Rx_E_Dx_PAM7_unused		: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0);
		Rx_E_Dx_PAM7_unused_n	: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0);						
		
-- ####################################################################
-- ##### USER STUFF 			#############################################
-- ####################################################################

		USER_PB					: in  std_logic_vector(0 downto 0);				
--		USER_DIP				: in  std_logic_vector(6 downto 0);				
		USER_LED				: out std_logic_vector(3 downto 0);
--		USER_IO				  : out std_logic_vector(9 downto 0);		

-- ####################################################################
-- ##### Simplex PHY AVMM Signals		################################
-- ####################################################################
		
--		reconfig_write          : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- write
--		reconfig_read           : in  std_logic_vector(0 downto 0)  := (others => 'X'); -- read
--		reconfig_address        : in  std_logic_vector(10 downto 0) := (others => 'X'); -- address
--		reconfig_writedata      : in  std_logic_vector(31 downto 0) := (others => 'X'); -- writedata
--		reconfig_readdata       : out std_logic_vector(31 downto 0);                    -- readdata
--		reconfig_waitrequest    : out std_logic_vector(0 downto 0);                     -- waitrequest

		reconfig_clk         		: in  std_logic_vector(0 downto 0)   := (others => '0')
		
	);
end entity XCVR_Top;

architecture rtl of XCVR_Top is

-- Reset Signals
	signal MasterReset         : std_logic_vector(0 downto 0):= (others => '1');
	signal ResetPB         		: std_logic:= '1';
	signal ResetPBn         	: std_logic:= '0';	

	signal PowerOnReset			: std_logic := '0';--_vector(0 downto 0)   := (others => '0');
	signal PowerOnReset_n		: std_logic := '1';--_vector(0 downto 0)   := (others => '1');
	signal ResetCount				: std_logic_vector(5 downto 0) := (OTHERS => '1');
	
	signal reconfig_write          : std_logic_vector(0 downto 0)  := (others => 'X'); -- write
	signal reconfig_read           : std_logic_vector(0 downto 0)  := (others => 'X'); -- read
	signal reconfig_address        : std_logic_vector(10 downto 0) := (others => 'X'); -- address
	signal reconfig_writedata      : std_logic_vector(31 downto 0) := (others => 'X'); -- writedata
	signal reconfig_readdata       : std_logic_vector(31 downto 0);                    -- readdata
	signal reconfig_waitrequest    : std_logic_vector(0 downto 0);                     -- waitrequest
	
-- ####################################################################
-- ##### internal reference clocks ####################################
-- ####################################################################
	
	signal E_Dx_NRZ0_refclk, E_Dx_NRZ1_refclk, E_Dx_NRZ2_refclk, E_Dx_NRZ3_refclk, E_Dx_NRZ4_refclk, E_Dx_NRZ5_refclk, E_Dx_NRZ6_refclk, E_Dx_NRZ7_refclk	: std_logic_vector(0 downto 0);					
	signal E_Dx_PAM0_refclk, E_Dx_PAM1_refclk, E_Dx_PAM2_refclk, E_Dx_PAM3_refclk, E_Dx_PAM4_refclk, E_Dx_PAM5_refclk, E_Dx_PAM6_refclk, E_Dx_PAM7_refclk		: std_logic_vector(0 downto 0);	
	
-- ####################################################################
-- ##### NRZ Duplex Channels 			#####################################
-- ####################################################################

component E_Dx_NRZ0_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ0_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ0_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ0_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ0_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ1_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ1_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ1_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ1_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ1_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ2_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ2_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ2_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ2_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ2_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ3_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ3_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ3_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ3_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ3_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ4_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ4_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ4_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ4_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ4_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ5_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ5_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ5_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ5_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ5_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ6_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ6_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ6_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ6_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ6_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_NRZ7_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="10G3";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_NRZ7_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_NRZ7_num-1 downto 0);		
		rx_serial_data		: in  std_logic_vector(E_Dx_NRZ7_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_NRZ7_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

-- ####################################################################
-- ##### PAM Duplex Channels 			###########################################
-- ####################################################################

component E_Dx_PAM0_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in  std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM0_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM0_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM1_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM1_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM1_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM2_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in  std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM2_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM2_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM3_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM3_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM3_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM4_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in  std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM4_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM4_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM5_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM5_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM5_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM6_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in  std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM6_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM6_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;

component E_Dx_PAM7_Wrap is
	generic(	NumChannels:integer:=0;
				Rate:string:="USR0";
				SIPatterns:boolean:=FALSE);					
	PORT(
		refclk						: in std_logic_vector(0 downto 0);
		tx_serial_data		: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);
		tx_serial_data_n	: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);	
		unusedtx					: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);
		unusedtx_n				: out std_logic_vector(E_Dx_PAM7_num-1 downto 0);	
		rx_serial_data		: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0)   := (others => '0'); 
		rx_serial_data_n	: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0)   := (others => '0'); 
		unusedrx					: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0)   := (others => '0'); 
		unusedrx_n				: in  std_logic_vector(E_Dx_PAM7_num-1 downto 0)   := (others => '0'); 
		reconfig_clk			: in  std_logic_vector(0 downto 0)   := (others => '0');
		reconfig_reset		: in  std_logic_vector(0 downto 0)   := (others => '0');
		phy_reset					: in  std_logic_vector(0 downto 0)   := (others => '0')		
		);
end component;


component E_refclk_select is
	generic(E_Dx_NRZ0_refclk_sel, E_Dx_NRZ1_refclk_sel, E_Dx_NRZ2_refclk_sel, E_Dx_NRZ3_refclk_sel, E_Dx_NRZ4_refclk_sel, E_Dx_NRZ5_refclk_sel, E_Dx_NRZ6_refclk_sel, E_Dx_NRZ7_refclk_sel :string:="R4NT";
			    E_Dx_PAM0_refclk_sel, E_Dx_PAM1_refclk_sel, E_Dx_PAM2_refclk_sel, E_Dx_PAM3_refclk_sel, E_Dx_PAM4_refclk_sel, E_Dx_PAM5_refclk_sel, E_Dx_PAM6_refclk_sel, E_Dx_PAM7_refclk_sel :string:="R4NT");
	port(           
			REFCLK_R9A0, REFCLK_R9A1, REFCLK_R9A2, REFCLK_R9A3, REFCLK_R9A4, REFCLK_R9A5, REFCLK_R9A6, REFCLK_R9A7, REFCLK_R9A8: in std_logic_vector(0 downto 0);		-- Right bank 9A
			REFCLK_R9B0, REFCLK_R9B1, REFCLK_R9B2, REFCLK_R9B3, REFCLK_R9B4, REFCLK_R9B5, REFCLK_R9B6, REFCLK_R9B7, REFCLK_R9B8: in std_logic_vector(0 downto 0);		-- Right bank 9B					
			REFCLK_R9C0, REFCLK_R9C1, REFCLK_R9C2, REFCLK_R9C3, REFCLK_R9C4, REFCLK_R9C5, REFCLK_R9C6, REFCLK_R9C7, REFCLK_R9C8: in std_logic_vector(0 downto 0);		-- Right bank 9C
			REFCLK_L8B0, REFCLK_L8B1, REFCLK_L8B2, REFCLK_L8B3, REFCLK_L8B4, REFCLK_L8B5, REFCLK_L8B6, REFCLK_L8B7, REFCLK_L8B8: in std_logic_vector(0 downto 0);		-- Left bank 8B					
			REFCLK_L8C0, REFCLK_L8C1, REFCLK_L8C2, REFCLK_L8C3, REFCLK_L8C4, REFCLK_L8C5, REFCLK_L8C6, REFCLK_L8C7, REFCLK_L8C8: in std_logic_vector(0 downto 0);		-- Left bank 8C
			
			E_Dx_NRZ0_refclk, E_Dx_NRZ1_refclk, E_Dx_NRZ2_refclk, E_Dx_NRZ3_refclk, E_Dx_NRZ4_refclk, E_Dx_NRZ5_refclk, E_Dx_NRZ6_refclk, E_Dx_NRZ7_refclk : out std_logic_vector(0 downto 0);
			E_Dx_PAM0_refclk, E_Dx_PAM1_refclk, E_Dx_PAM2_refclk, E_Dx_PAM3_refclk, E_Dx_PAM4_refclk, E_Dx_PAM5_refclk, E_Dx_PAM6_refclk, E_Dx_PAM7_refclk : out std_logic_vector(0 downto 0)
	);
end component;
                                                         
begin                                                                                                                                                                                            
-- ################################################################################################################
-- ##### Start RTL Here ###########################################################################################
-- ################################################################################################################


-- ####################################################################
-- ##### NRZ Duplex Channels 			#####################################
-- ####################################################################

Gen_E_Dx_NRZ0:if E_Dx_NRZ0_num > 0 generate
	E_Dx_NRZ0: E_Dx_NRZ0_Wrap
	Generic Map(E_Dx_NRZ0_num,	
					E_Dx_NRZ0_Rate,
					E_Dx_NRZ0_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ0_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ0  (E_Dx_NRZ0_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ0_n(E_Dx_NRZ0_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ0  (E_Dx_NRZ0_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ0_n(E_Dx_NRZ0_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_NRZ0;

Gen_E_Dx_NRZ1:if E_Dx_NRZ1_num > 0 generate
	E_Dx_NRZ1: E_Dx_NRZ1_Wrap
	Generic Map(E_Dx_NRZ1_num,	
					E_Dx_NRZ1_Rate,
					E_Dx_NRZ1_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ1_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ1  (E_Dx_NRZ1_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ1_n(E_Dx_NRZ1_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ1  (E_Dx_NRZ1_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ1_n(E_Dx_NRZ1_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			

		); 
end generate Gen_E_Dx_NRZ1;

Gen_E_Dx_NRZ2:if E_Dx_NRZ2_num > 0 generate
	E_Dx_NRZ2: E_Dx_NRZ2_Wrap
	Generic Map(E_Dx_NRZ2_num,	
					E_Dx_NRZ2_Rate,
					E_Dx_NRZ2_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ2_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ2  (E_Dx_NRZ2_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ2_n(E_Dx_NRZ2_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ2  (E_Dx_NRZ2_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ2_n(E_Dx_NRZ2_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_NRZ2;

Gen_E_Dx_NRZ3:if E_Dx_NRZ3_num > 0 generate
	E_Dx_NRZ3: E_Dx_NRZ3_Wrap
	Generic Map(E_Dx_NRZ3_num,	
					E_Dx_NRZ3_Rate,
					E_Dx_NRZ3_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ3_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ3  (E_Dx_NRZ3_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ3_n(E_Dx_NRZ3_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ3  (E_Dx_NRZ3_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ3_n(E_Dx_NRZ3_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			

		); 
end generate Gen_E_Dx_NRZ3;

Gen_E_Dx_NRZ4:if E_Dx_NRZ4_num > 0 generate
	E_Dx_NRZ4: E_Dx_NRZ4_Wrap
	Generic Map(E_Dx_NRZ4_num,	
					E_Dx_NRZ4_Rate,
					E_Dx_NRZ4_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ4_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ4  (E_Dx_NRZ4_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ4_n(E_Dx_NRZ4_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ4  (E_Dx_NRZ4_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ4_n(E_Dx_NRZ4_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_NRZ4;

Gen_E_Dx_NRZ5:if E_Dx_NRZ5_num > 0 generate
	E_Dx_NRZ5: E_Dx_NRZ1_Wrap
	Generic Map(E_Dx_NRZ5_num,	
					E_Dx_NRZ5_Rate,
					E_Dx_NRZ5_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ5_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ5  (E_Dx_NRZ5_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ5_n(E_Dx_NRZ5_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ5  (E_Dx_NRZ5_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ5_n(E_Dx_NRZ5_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			

		); 
end generate Gen_E_Dx_NRZ5;

Gen_E_Dx_NRZ6:if E_Dx_NRZ6_num > 0 generate
	E_Dx_NRZ6: E_Dx_NRZ6_Wrap
	Generic Map(E_Dx_NRZ6_num,	
					E_Dx_NRZ6_Rate,
					E_Dx_NRZ6_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ6_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ6  (E_Dx_NRZ6_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ6_n(E_Dx_NRZ6_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ6  (E_Dx_NRZ6_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ6_n(E_Dx_NRZ6_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_NRZ6;

Gen_E_Dx_NRZ7:if E_Dx_NRZ7_num > 0 generate
	E_Dx_NRZ7: E_Dx_NRZ7_Wrap
	Generic Map(E_Dx_NRZ7_num,	
					E_Dx_NRZ7_Rate,
					E_Dx_NRZ7_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_NRZ7_refclk,
			tx_serial_data  	=> Tx_E_Dx_NRZ7  (E_Dx_NRZ7_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_NRZ7_n(E_Dx_NRZ7_num-1 downto 0),			
			rx_serial_data  	=> Rx_E_Dx_NRZ7  (E_Dx_NRZ7_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_NRZ7_n(E_Dx_NRZ7_num-1 downto 0),			
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			

		); 
end generate Gen_E_Dx_NRZ7;
                     
                
-- ####################################################################
-- ##### PAM Duplex x1 Channels 			###########################################
-- ####################################################################
                     
Gen_E_Dx_PAM0:if E_Dx_PAM0_num > 0 generate
	E_Dx_PAM0: E_Dx_PAM0_Wrap
	Generic Map(E_Dx_PAM0_num,	
							E_Dx_PAM0_Rate,
							E_Dx_PAM0_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_PAM0_refclk,
			tx_serial_data  	=> Tx_E_Dx_PAM0				(E_Dx_PAM0_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_PAM0_n			(E_Dx_PAM0_num-1 downto 0),	
			unusedtx  			=> Tx_E_Dx_PAM0_unused 	(E_Dx_PAM0_num-1 downto 0),
			unusedtx_n  		=> Tx_E_Dx_PAM0_unused_n	(E_Dx_PAM0_num-1 downto 0),						
			rx_serial_data  	=> Rx_E_Dx_PAM0				(E_Dx_PAM0_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_PAM0_n			(E_Dx_PAM0_num-1 downto 0),
			unusedrx  			=> Rx_E_Dx_PAM0_unused	(E_Dx_PAM0_num-1 downto 0),
			unusedrx_n  		=> Rx_E_Dx_PAM0_unused_n	(E_Dx_PAM0_num-1 downto 0),							
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_PAM0;


Gen_E_Dx_PAM1:if E_Dx_PAM1_num > 0 generate
	E_Dx_PAM1: E_Dx_PAM1_Wrap
	Generic Map(E_Dx_PAM1_num,	
							E_Dx_PAM1_Rate,
							E_Dx_PAM1_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_PAM1_refclk,
			tx_serial_data  	=> Tx_E_Dx_PAM1				(E_Dx_PAM1_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_PAM1_n			(E_Dx_PAM1_num-1 downto 0),	
			unusedtx  			=> Tx_E_Dx_PAM1_unused	(E_Dx_PAM1_num-1 downto 0),
			unusedtx_n  		=> Tx_E_Dx_PAM1_unused_n	(E_Dx_PAM1_num-1 downto 0),						
			rx_serial_data  	=> Rx_E_Dx_PAM1				(E_Dx_PAM1_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_PAM1_n			(E_Dx_PAM1_num-1 downto 0),
			unusedrx  			=> Rx_E_Dx_PAM1_unused	(E_Dx_PAM1_num-1 downto 0),
			unusedrx_n  		=> Rx_E_Dx_PAM1_unused_n	(E_Dx_PAM1_num-1 downto 0),							
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_PAM1;


Gen_E_Dx_PAM2:if E_Dx_PAM2_num > 0 generate
	E_Dx_PAM2: E_Dx_PAM2_Wrap
	Generic Map(E_Dx_PAM2_num,	
							E_Dx_PAM2_Rate,
							E_Dx_PAM2_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_PAM2_refclk,
			tx_serial_data  	=> Tx_E_Dx_PAM2				(E_Dx_PAM2_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_PAM2_n			(E_Dx_PAM2_num-1 downto 0),	
			unusedtx  			=> Tx_E_Dx_PAM2_unused 	(E_Dx_PAM2_num-1 downto 0),
			unusedtx_n  		=> Tx_E_Dx_PAM2_unused_n	(E_Dx_PAM2_num-1 downto 0),						
			rx_serial_data  	=> Rx_E_Dx_PAM2				(E_Dx_PAM2_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_PAM2_n			(E_Dx_PAM2_num-1 downto 0),
			unusedrx  			=> Rx_E_Dx_PAM2_unused	(E_Dx_PAM2_num-1 downto 0),
			unusedrx_n  		=> Rx_E_Dx_PAM2_unused_n	(E_Dx_PAM2_num-1 downto 0),							
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_PAM2;


Gen_E_Dx_PAM3:if E_Dx_PAM3_num > 0 generate
	E_Dx_PAM3: E_Dx_PAM3_Wrap
	Generic Map(E_Dx_PAM3_num,	
							E_Dx_PAM3_Rate,
							E_Dx_PAM3_EnSIPatterns)
	    port map
	    (   
			refclk		    	=> E_Dx_PAM3_refclk,
			tx_serial_data  	=> Tx_E_Dx_PAM3				(E_Dx_PAM3_num-1 downto 0),
			tx_serial_data_n  => Tx_E_Dx_PAM3_n			(E_Dx_PAM3_num-1 downto 0),	
			unusedtx  			=> Tx_E_Dx_PAM3_unused	(E_Dx_PAM3_num-1 downto 0),
			unusedtx_n  		=> Tx_E_Dx_PAM3_unused_n	(E_Dx_PAM3_num-1 downto 0),						
			rx_serial_data  	=> Rx_E_Dx_PAM3				(E_Dx_PAM3_num-1 downto 0),
			rx_serial_data_n  => Rx_E_Dx_PAM3_n			(E_Dx_PAM3_num-1 downto 0),
			unusedrx  			=> Rx_E_Dx_PAM3_unused	(E_Dx_PAM3_num-1 downto 0),
			unusedrx_n  		=> Rx_E_Dx_PAM3_unused_n	(E_Dx_PAM3_num-1 downto 0),							
			reconfig_clk      => reconfig_clk,
			reconfig_reset    => MasterReset,
			phy_reset        	=> MasterReset			
		); 
end generate Gen_E_Dx_PAM3;


-- ####################################################################                 
-- ##### Power On Reset 			#########################################
-- ####################################################################                 

process(reconfig_clk)
begin
	if rising_edge (reconfig_clk(0)) then
		if ResetCount = 0 then
			PowerOnReset <= '0';
		else
			PowerOnReset <= '1';
		  	ResetCount <= ResetCount - 1;
		end if;
	end if;
end process;

PowerOnReset_n <= not PowerOnReset;	
--ResetPB <= not USER_PB(0);	--invert pushbutton	
ResetPB <= not '1';	--invert pushbutton	
MasterReset(0) <= PowerOnReset OR ResetPB;  -- Allow POR and PB operation


refsel:E_refclk_select
	generic map(E_Dx_NRZ0_refclk_sel, E_Dx_NRZ1_refclk_sel, E_Dx_NRZ2_refclk_sel, E_Dx_NRZ3_refclk_sel, E_Dx_NRZ4_refclk_sel, E_Dx_NRZ5_refclk_sel, E_Dx_NRZ6_refclk_sel, E_Dx_NRZ7_refclk_sel,
					E_Dx_PAM0_refclk_sel, E_Dx_PAM1_refclk_sel, E_Dx_PAM2_refclk_sel, E_Dx_PAM3_refclk_sel, E_Dx_PAM4_refclk_sel, E_Dx_PAM5_refclk_sel, E_Dx_PAM6_refclk_sel, E_Dx_PAM7_refclk_sel)
	port map(  
			-- Right bank 9A         
			REFCLK_R9A0	=> REFCLK_R9A0, 
			REFCLK_R9A1	=> REFCLK_R9A1, 
			REFCLK_R9A2	=> REFCLK_R9A2, 
			REFCLK_R9A3	=> REFCLK_R9A3, 
			REFCLK_R9A4	=> REFCLK_R9A4, 
			REFCLK_R9A5	=> REFCLK_R9A5, 
			REFCLK_R9A6	=> REFCLK_R9A6, 
			REFCLK_R9A7	=> REFCLK_R9A7, 
			REFCLK_R9A8	=> REFCLK_R9A8,
			
			-- Right bank 9B								
			REFCLK_R9B0	=> REFCLK_R9B0, 
			REFCLK_R9B1	=> REFCLK_R9B1, 
			REFCLK_R9B2	=> REFCLK_R9B2, 
			REFCLK_R9B3	=> REFCLK_R9B3, 
			REFCLK_R9B4	=> REFCLK_R9B4, 
			REFCLK_R9B5	=> REFCLK_R9B5, 
			REFCLK_R9B6	=> REFCLK_R9B6, 
			REFCLK_R9B7	=> REFCLK_R9B7, 
			REFCLK_R9B8	=> REFCLK_R9B8,
			
			-- Right bank 9C			
			REFCLK_R9C0 => REFCLK_R9C0, 
			REFCLK_R9C1 => REFCLK_R9C1, 
			REFCLK_R9C2 => REFCLK_R9C2, 
			REFCLK_R9C3 => REFCLK_R9C3, 
			REFCLK_R9C4 => REFCLK_R9C4, 
			REFCLK_R9C5 => REFCLK_R9C5, 
			REFCLK_R9C6 => REFCLK_R9C6, 
			REFCLK_R9C7 => REFCLK_R9C7, 
			REFCLK_R9C8 => REFCLK_R9C8,
			
			-- Left bank 8B								
			REFCLK_L8B0 => REFCLK_L8B0,
			REFCLK_L8B1 => REFCLK_L8B1,
			REFCLK_L8B2 => REFCLK_L8B2,
			REFCLK_L8B3 => REFCLK_L8B3,
			REFCLK_L8B4 => REFCLK_L8B4,
			REFCLK_L8B5 => REFCLK_L8B5,
			REFCLK_L8B6 => REFCLK_L8B6,
			REFCLK_L8B7 => REFCLK_L8B7,
			REFCLK_L8B8 => REFCLK_L8B8,
			
			-- Left bank 8C			
			REFCLK_L8C0 => REFCLK_L8C0, 
			REFCLK_L8C1 => REFCLK_L8C1, 
			REFCLK_L8C2 => REFCLK_L8C2, 
			REFCLK_L8C3 => REFCLK_L8C3, 
			REFCLK_L8C4 => REFCLK_L8C4, 
			REFCLK_L8C5 => REFCLK_L8C5, 
			REFCLK_L8C6 => REFCLK_L8C6, 
			REFCLK_L8C7 => REFCLK_L8C7, 
			REFCLK_L8C8 => REFCLK_L8C8,
			
			E_Dx_NRZ0_refclk => E_Dx_NRZ0_refclk, 
			E_Dx_NRZ1_refclk => E_Dx_NRZ1_refclk, 
			E_Dx_NRZ2_refclk => E_Dx_NRZ2_refclk,
			E_Dx_NRZ3_refclk => E_Dx_NRZ3_refclk,
			E_Dx_NRZ4_refclk => E_Dx_NRZ4_refclk,
			E_Dx_NRZ5_refclk => E_Dx_NRZ5_refclk,
			E_Dx_NRZ6_refclk => E_Dx_NRZ6_refclk,
			E_Dx_NRZ7_refclk => E_Dx_NRZ7_refclk,
			
			E_Dx_PAM0_refclk => E_Dx_PAM0_refclk, 
			E_Dx_PAM1_refclk => E_Dx_PAM1_refclk,
			E_Dx_PAM2_refclk => E_Dx_PAM2_refclk,
			E_Dx_PAM3_refclk => E_Dx_PAM3_refclk,
			E_Dx_PAM4_refclk => E_Dx_PAM4_refclk,
			E_Dx_PAM5_refclk => E_Dx_PAM5_refclk,
			E_Dx_PAM6_refclk => E_Dx_PAM6_refclk,
			E_Dx_PAM7_refclk => E_Dx_PAM7_refclk 
	);
end rtl;
